@@ -6,6 +6,8 @@ from .psf import Gaussian2D, Moffat2D, FastGaussian, KeepGoodStars, FWHM
 from .centroids import BalletCentroid
 from .photometry import PhotutilsAperturePhotometry, SEAperturePhotometry, PhotutilsPSFPhotometry
 from .imutils import *
+from .shepard import  Shepard
+from ..utils import register_args
 
 
 # import prose
