@@ -9,6 +9,7 @@ from photutils import CircularAperture, CircularAnnulus
 from .. import FitsManager
 from photutils.psf import IntegratedGaussianPRF, DAOGroup, BasicPSFPhotometry
 from ..core import Block
+from ..utils import register_args
 try:
     import sep
 except:
@@ -21,6 +22,7 @@ except:
 
 class PhotutilsPSFPhotometry(Block):
 
+    @register_args
     def __init__(self, fwhm, **kwargs):
         super().__init__(**kwargs)
 
@@ -53,6 +55,20 @@ class PhotutilsAperturePhotometry(Block):
     Aperture photometry using :code:`photutils`.
     For more details check https://photutils.readthedocs.io/en/stable/aperture.html
 
+    |write| 
+    
+    - ``Image.stars_coords``
+    - ``Image.apertures_area``
+    - ``Image.sky``
+    - ``Image.fluxes``
+    - ``Image.annulus_area``
+    - ``Image.annulus_rin``
+    - ``Image.annulus_rout``
+    - ``Image.apertures_radii``
+    - ``Image.fluxes``
+
+    |modify| 
+
     Parameters
     ----------
     apertures : ndarray or list, optional
@@ -63,6 +79,7 @@ class PhotutilsAperturePhotometry(Block):
         radius of the outer annulus in fraction of fwhm, by default 8
     """
 
+    @register_args
     def __init__(
             self,
             apertures=None,
@@ -154,11 +171,17 @@ class PhotutilsAperturePhotometry(Block):
         data[data < 0] = 0
 
         photometry = aperture_photometry(data, self.circular_apertures)
-        image.fluxes = np.array([
+        fluxes = np.array([
             photometry[f"aperture_sum_{a}"] - (bkg_median * self.circular_apertures_area[a])
             for a in range(len(self.apertures))
         ])
 
+        # dummy values if negative or nan
+        fluxes[np.isnan(fluxes)] = 1
+        fluxes[fluxes < 0 ] = 1
+
+        image.fluxes = fluxes
+
         self.compute_error(image)
         image.header["sky"] = np.mean(image.sky)
 
@@ -201,6 +224,7 @@ class SEAperturePhotometry(Block):
         radius of the outer annulus in fraction of fwhm, by default 8
     """
 
+    @register_args
     def __init__(self, apertures=None, r_in=5, r_out=8, fwhm_scale=True, **kwargs):
 
         super().__init__(**kwargs)
