@@ -8,8 +8,9 @@ from photutils.psf import extract_stars
 from astropy.stats import gaussian_sigma_to_fwhm
 from ..core import Block
 import matplotlib.pyplot as plt
-from scipy.stats import shapiro
-
+from collections import OrderedDict
+from ..utils import fast_binning
+from ..utils import register_args
 
 def image_psf(image, stars, size=15, normalize=False, return_cutouts=False):
     """
@@ -77,6 +78,34 @@ def cutouts(image, stars, size=15):
         return stars
 
 
+def good_cutouts(image, xy, r=30, upper=40000, lower=1000, trim=100):
+    idxs, _cuts = cutouts(image, xy, r)
+    cuts = OrderedDict(zip(idxs, _cuts))
+    peaks = [cutout.data.max() for cutout in cuts.values()]
+
+    for i, cutout in cuts.copy().items():
+        if i in cuts:
+            peak = cutout.data.max()
+            center = cutout.center
+
+            # removing saturated and faint stars
+            if peak > upper or peak < lower:
+                del cuts[i]
+
+            # removing stars on borders
+            elif np.any(center < [trim, trim]) or np.any(center > np.array(image.shape) - trim):
+                del cuts[i]
+
+            # removing close stars
+            closest = idxs[np.nonzero(np.linalg.norm(center - xy[idxs], axis=1) < r)[0]]
+            if len(closest) > 1:
+                for j in closest:
+                    if j in cuts:
+                        del cuts[j]
+
+    return cuts
+
+
 def moments(data):
     """Returns (height, x, y, width_x, width_y)
     the gaussian parameters of a 2D distribution by calculating its
@@ -153,34 +182,31 @@ class PSFModel(Block):
         return self.optimize()
 
 
-class FWHM(Block):
+class FWHM(PSFModel):
     """
-    Fast empirical FWHM (courtesy of Arielle Bertrou-Cantou)
+    Fast empirical FWHM (based on Arielle Bertrou-Cantou's idea)
     """
+    
+    @register_args
+    def __init__(self, cutout_size=51, **kwargs):
+        super().__init__(cutout_size=cutout_size, **kwargs)
+        Y, X = np.indices((self.cutout_size,self.cutout_size))
+        x = y = self.cutout_size/2
+        self.radii = (np.sqrt((X - x) ** 2 + (Y - y) ** 2)).flatten()
 
-    def __init__(self, n=15, **kwargs):
-        super().__init__(**kwargs)
-        self.n = n
-
-    def run(self, image, **kwargs):
-        psf = image_psf(image.data, image.stars_coords, self.n)
+    def optimize(self):
+        psf = self.epsf.copy()
         psf -= np.min(psf)
-        mask = psf > np.max(psf) / 2
-        r = np.sqrt(np.sum(mask) / np.pi) * 2
-        image.fwhm = r
-        image.fwhmx = r
-        image.fwhmy = r
-        image.header["FWHM"] = image.fwhm
-        image.header["FWHMX"] = image.fwhmx
-        image.header["FWHMY"] = image.fwhmy
-        image.header["PSFANGLE"] = 0
-        image.header["FWHMALG"] = self.__class__.__name__
-
+        pixels = psf.flatten()
+        binned_radii, binned_pixels, _ = fast_binning(self.radii, pixels, bins=1)
+        fwhm = 2*binned_radii[np.flatnonzero(binned_pixels > np.max(binned_pixels)/2)[-1]]
+        return fwhm, fwhm, 0
 
 class FastGaussian(PSFModel):
     """
     Fit a symetric 2D Gaussian model to an image effective PSF
     """
+    @register_args
     def __init__(self, cutout_size=21, **kwargs):
         super().__init__(cutout_size=cutout_size, **kwargs)
 
@@ -210,9 +236,28 @@ class FastGaussian(PSFModel):
 
 
 class Gaussian2D(PSFModel):
-    """
+    r"""
     Fit an elliptical 2D Gaussian model to an image effective PSF
+
+    Model is
+
+    .. math::
+
+        f(x, y|A, x_0, y_0, \sigma_x, \sigma_y, \theta, b) = - A \exp\left(\frac{(x'-x'_0)^2}{2\sigma_x^2} \frac{(y'-y'_0)^2}{2\sigma_y^2}\right) + b
+
+    .. math::
+
+        \text{with}\quad \begin{gather*}
+        x' = xcos(\theta) + ysin(\theta) \\
+        y' = -xsin(\theta) + ycos(\theta)
+        \end{gather*}
+
+
+    is fitted from an effective psf. :code:`scipy.optimize.minimize` is used to minimize :math:`\chi ^2` from data. Initial parameters are found using the moments of the `effective psf <https://photutils.readthedocs.io/en/stable/epsf.html>`_. This method is 4 times faster than :code:`photutils.centroids.fit_2dgaussian` and lead to similar results.
+
     """
+
+    @register_args
     def __init__(self, cutout_size=21, **kwargs):
         super().__init__(cutout_size=cutout_size, **kwargs)
 
@@ -253,6 +298,8 @@ class Moffat2D(PSFModel):
     """
     Fit an elliptical 2D Moffat model to an image effective PSF
     """
+
+    @register_args
     def __init__(self, cutout_size=21, **kwargs):
         super().__init__(cutout_size=cutout_size, **kwargs)
 
@@ -297,6 +344,7 @@ class Moffat2D(PSFModel):
 
 class KeepGoodStars(Block):
 
+    @register_args
     def __init__(self, n=-1, **kwargs):
         super().__init__(**kwargs)
         self.n = n
