@@ -1,9 +1,10 @@
-from .. import Sequence, blocks, Block, Image
+from .. import Sequence, MultiProcessSequence, blocks, Block, Image
 import os
 from os import path
 from pathlib import Path
-import xarray as xr
+from .. import utils
 from .photometry import plot_function
+from astropy.time import Time
 
 
 class Calibration:
@@ -14,10 +15,13 @@ class Calibration:
 
     Parameters
     ----------
-    destination : str, optional
-        Destination of the newly created folder, by default beside the folder given to FitsManager
-    reference : float, optional
-        Reference image to use for alignment from 0 (first image) to 1 (last image), by default 1/2
+    reference : float or str, optional
+        Reference image to use for alignment:
+         
+        - if ``float``: from 0 (first image) to 1 (last image)
+        - if ``str``: path of the reference image
+        
+        by default 1/2
     overwrite : bool, optional
         whether to overwrite existing products, by default False
     flats : list, optional
@@ -29,7 +33,9 @@ class Calibration:
     images : list, optional
         list of images paths to be calibrated, by default None
     psf : `Block`, optional
-        a `Block` to be used to characterise the effective psf, by default blocks.Moffat2D
+        a `Block` to be used to characterise the effective psf, by default None, setting a default blocks.Moffat2D
+    detection: `Block`, optional
+        a `Block` to be used for stars detection, by default None, setting a default blocks.SegmentedPeaks
     show: bool, optional
         within a notebook, whether to show processed image during reduction, by default False 
     verbose: bool, optional
@@ -50,12 +56,14 @@ class Calibration:
             bias=None,
             darks=None,
             images=None,
-            psf=blocks.Moffat2D,
+            psf=None,
+            detection=None,
             verbose=True,
             show=False,
             twirl=True,
             n=None,
             loader=Image,
+            cores=False,
     ):
         self.destination = None
         self.overwrite = overwrite
@@ -64,6 +72,8 @@ class Calibration:
         self.show = show
         self.n = n if n is not None else (12 if twirl else 50)
         self.twirl = twirl
+        self.cores = cores
+        self.xarray = None
 
         if show:
             self.show = blocks.LivePlot(plot_function, size=(10, 10))
@@ -79,12 +89,31 @@ class Calibration:
         self.detection_s = None
         self.calibration_s = None
 
-        assert psf is None or issubclass(psf, Block), "psf must be a subclass of Block"
+        # Checking psf block
+        if psf is None:
+            psf = blocks.Moffat2D(name="fwhm")
+        else:
+            assert isinstance(psf, Block), "psf must be a subclass of Block"
+            psf.name = "fwhm"
+
         self.psf = psf
-        
+
+        # checking detection block
+        if detection is None:
+            detection = blocks.SegmentedPeaks(n_stars=self.n, name="detection")
+        else:
+            assert isinstance(detection, Block), "psf must be a subclass of Block"
+            detection.name = "detection"
+
+        self.detection = detection
+
         # set reference file
-        reference_id = int(self._reference * len(self._images))
-        self.reference_fits = self._images[reference_id]
+        if isinstance(self._reference, (int, float)):
+            reference_id = int(self._reference * len(self._images))
+            self.reference_fits = self._images[reference_id]
+        elif isinstance(self._reference, (str, Path)):
+            self.reference_fits = self._reference
+
         self.calibration_block = blocks.Calibration(self.darks, self.flats, self.bias, loader=loader, name="calibration")
 
     def run(self, destination, gif=True):
@@ -103,28 +132,27 @@ class Calibration:
         self.detection_s = Sequence([
             self.calibration_block,
             blocks.Trim(name="trimming"),
-            blocks.SegmentedPeaks(n_stars=self.n, name="detection"),
+            self.detection,
             blocks.ImageBuffer(name="buffer")
-        ], self.reference_fits, loader=self.loader)
+        ], loader=self.loader)
 
-        self.detection_s.run(show_progress=False)
+        self.detection_s.run(self.reference_fits, show_progress=False)
 
         ref_image = self.detection_s.buffer.image
         ref_stars = ref_image.stars_coords
 
-        self.calibration_s = Sequence([
+        if self.twirl:
+            assert len(ref_stars) >= 4, f"Only {len(ref_stars)} stars detected (must be >= 4). See detection kwargs"
+
+        SequenceObject = MultiProcessSequence if self.cores else Sequence
+
+        self.calibration_s = SequenceObject([
             self.calibration_block,
             blocks.Trim(name="trimming", skip_wcs=True),
             blocks.Flip(ref_image, name="flip"),
-            blocks.SegmentedPeaks(n_stars=self.n, name="detection"),
+            self.detection,
             blocks.Twirl(ref_stars, n=self.n, name="twirl") if self.twirl else blocks.XYShift(ref_stars),
-            self.psf(name="fwhm"),
-            blocks.Cutout2D(ref_image) if not self.twirl else blocks.Pass(),
-            blocks.SaveReduced(self.destination, overwrite=self.overwrite, name="save_reduced"),
-            blocks.AffineTransform(stars=True, data=True) if self.twirl else blocks.Pass(),
-            self.show,
-            blocks.Stack(self.stack_path, header=ref_image.header, overwrite=self.overwrite, name="stack"),
-            blocks.Video(self.gif_path, name="video", from_fits=True),
+            self.psf,
             blocks.XArray(
                 ("time", "jd_utc"),
                 ("time", "bjd_tdb"),
@@ -135,27 +163,57 @@ class Calibration:
                 ("time", "dx"),
                 ("time", "dy"),
                 ("time", "airmass"),
-                ("time", "exposure")
-            )
-        ], self._images, name="Calibration", loader=self.loader)
+                ("time", "exposure"),
+                ("time", "path")
+            ),
+            blocks.Cutout2D(ref_image) if not self.twirl else blocks.Pass(),
+            blocks.SaveReduced(self.destination, overwrite=self.overwrite, name="save_reduced"),
+            blocks.AffineTransform(stars=True, data=True) if self.twirl else blocks.Pass(),
+            self.show,
+            blocks.Stack(self.stack_path, header=ref_image.header, overwrite=self.overwrite, name="stack"),
+            gif_block,
+        ], name="Calibration", loader=self.loader)
 
-        self.calibration_s.run(show_progress=self.verbose)
+        self.calibration_s.run(self._images, show_progress=self.verbose)
 
         # saving xarray
-        calib_xarray = self.calibration_s.xarray.xarray
-        stack_xarray = self.calibration_s.stack.xarray
-        xarray = xr.merge([calib_xarray, stack_xarray], combine_attrs="no_conflicts")
+        xarray = self.calibration_s.xarray.xarray
+        # first image serve as reference for info (not reference image because it
+        # can be from another observation (we encountered this use case)
+        reference = self.loader(self._images[0])
+        reference_header = reference.header
+        reference_telescope = reference.telescope
+        xarray.attrs.update(utils.header_to_cdf4_dict(reference_header))
+        xarray.attrs.update(dict(
+            target=-1,
+            aperture=-1,
+            telescope=reference_telescope.name,
+            filter=reference_header.get(reference_telescope.keyword_filter, ""),
+            exptime=reference_header.get(reference_telescope.keyword_exposure_time, ""),
+            name=reference_header.get(reference_telescope.keyword_object, ""),
+        ))
+
+        if reference_telescope.keyword_observation_date in reference_header:
+            date = reference_header[reference_telescope.keyword_observation_date]
+        else:
+            date = Time(reference_header[reference_telescope.keyword_jd], format="jd").datetime
+
+        xarray.attrs.update(dict(date=utils.format_iso_date(date).isoformat()))
+        xarray.coords["stack"] = (('w', 'h'), self.calibration_s.stack.stack)
+
         xarray = xarray.assign_coords(time=xarray.jd_utc)
+        xarray = xarray.sortby("time")
         xarray.attrs["time_format"] = "jd_utc"
         xarray.attrs["reduction"] = [b.__class__.__name__ for b in self.calibration_s.blocks]
-        xarray.to_netcdf(self.phot_path)
+        self.xarray = xarray
+        xarray.to_netcdf(self.phot)
 
     @property
     def stack_path(self):
         return self.destination / "stack.fits"
 
     @property
-    def phot_path(self):
+    def phot(self):
         return self.destination / (self.destination.name + ".phot")
 
     @property
@@ -182,9 +240,4 @@ class Calibration:
             os.mkdir(self.destination)
 
     def __repr__(self):
-        return f"{self.detection_s}\n{self.calibration_s}"
-
-    @property
-    def xarray(self):
-        return self.calibration_s.xarray()
-
+        return f"{self.detection_s}\n{self.calibration_s}"
\ No newline at end of file
